package mongodb

import (
	. "code.hoteas.com/golang/hotime/common"
	"context"
	"fmt"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type MongoDb struct {
	Client   *mongo.Client
	Ctx      context.Context
	DataBase *mongo.Database
	Connect  *mongo.Collection
	LastErr  error
}

func GetMongoDb(database, url string) (*MongoDb, error) {
	db := MongoDb{}
	clientOptions := options.Client().ApplyURI(url)

	db.Ctx = context.TODO()
	// Connect to MongoDb
	var err error
	db.Client, err = mongo.Connect(db.Ctx, clientOptions)
	if err != nil {
		return nil, err
	}
	// Check the connection
	err = db.Client.Ping(db.Ctx, nil)
	if err != nil {
		return nil, err
	}
	fmt.Println("Connected to MongoDb!")
	//databases, err := db.Client.ListDatabaseNames(db.Ctx, bson.M{})
	//if err != nil {
	//	return nil, err
	//}
	//fmt.Println(databases)
	db.DataBase = db.Client.Database(database)
	return &db, nil

}

func (that *MongoDb) Insert(table string, data interface{}) string {
	collection := that.DataBase.Collection(table)
	re, err := collection.InsertOne(that.Ctx, data)
	if err != nil {
		that.LastErr = err
		return ""
	}
	return ObjToStr(re.InsertedID)
}

func (that *MongoDb) InsertMany(table string, data ...interface{}) Slice {
	collection := that.DataBase.Collection(table)
	re, err := collection.InsertMany(that.Ctx, data)
	if err != nil {
		that.LastErr = err
		return Slice{}
	}

	return ObjToSlice(re.InsertedIDs)

}

func (that *MongoDb) Update(table string, data Map, where Map) int64 {
	collection := that.DataBase.Collection(table)
	re, err := collection.UpdateMany(that.Ctx, where, data)

	if err != nil {
		that.LastErr = err
		return 0
	}

	return re.ModifiedCount

}

func (that *MongoDb) Delete(table string, where Map) int64 {
	collection := that.DataBase.Collection(table)
	re, err := collection.DeleteMany(that.Ctx, where)

	if err != nil {
		that.LastErr = err
		return 0
	}

	return re.DeletedCount

}

func (that *MongoDb) Get(table string, where Map) Map {
	results := []Map{}
	var cursor *mongo.Cursor
	var err error
	collection := that.DataBase.Collection(table)
	if cursor, err = collection.Find(that.Ctx, where, options.Find().SetSkip(0), options.Find().SetLimit(2)); err != nil {
		that.LastErr = err
		return nil
	}
	//延迟关闭游标
	defer func() {
		if err := cursor.Close(that.Ctx); err != nil {
			that.LastErr = err

		}
	}()
	//这里的结果遍历可以使用另外一种更方便的方式：
	if err = cursor.All(that.Ctx, &results); err != nil {
		that.LastErr = err
		return nil
	}
	if len(results) > 0 {

		return results[0]
	}
	return nil
}

func (that MongoDb) Select(table string, where Map, page, pageRow int64) []Map {
	page = (page - 1) * pageRow
	if page < 0 {
		page = 0
	}
	results := []Map{}
	var cursor *mongo.Cursor
	var err error
	collection := that.DataBase.Collection(table)
	if cursor, err = collection.Find(that.Ctx, where, options.Find().SetSkip(page), options.Find().SetLimit(pageRow)); err != nil {
		that.LastErr = err
		return results
	}
	//延迟关闭游标
	defer func() {
		if err := cursor.Close(that.Ctx); err != nil {
			that.LastErr = err

		}
	}()
	//这里的结果遍历可以使用另外一种更方便的方式：

	if err = cursor.All(that.Ctx, &results); err != nil {
		that.LastErr = err
		return results
	}
	return results
}
